const Discord = require('discord.js');
const client = new Discord.Client();
const config = require('./config.json');
const dbconfig = require('./dbconfig.json');
const ready = require('./events/ready');
const message = require('./events/message');
const memberadd = require('./events/memberadd');
const memberremove = require('./events/memberremove');
const messagedelete = require('./events/messagedelete');
const fs = require('fs');
const mysql = require('mysql');
const messageupdate = require('./events/messageupdate');
const channelcreate = require('./events/channelcreate');
const channeldelete = require('./events/channeldelete');

const connection = mysql.createConnection({
    host     : dbconfig.host,
    user     : dbconfig.user,
    password : dbconfig.password,
    database : dbconfig.dbname
});

connection.connect((err)=> {
    if (err){
        throw err;
    }
    console.log('Successfully connected to MySQL database.');
});

client.commands = new Discord.Collection();
client.aliases = new Discord.Collection();
ready.ready(client);
message.message(client, config, Discord, connection);
memberadd.guildmemberadd(client, connection);
memberremove.guildmemberremove(client, connection);
messagedelete.messagedelete(client, connection, Discord);
messageupdate.messageupdate(client, connection, Discord);
channelcreate.channelcreate(client, connection, Discord);
channeldelete.channeldelete(client, connection, Discord);

fs.readdir("./commands/", (err, files) => {

    if (err) console.error(err);
    let jsfiles = files.filter(f => f.split(".").pop() === "js");

    if (jsfiles.length < 1) return console.log("I cannot find any command to run.");

    console.log(`Loading ${jsfiles.length} commands.`);
    jsfiles.forEach((f, i) => {
        let props = require(`./commands/${f}`);
        console.log(`${i + 1}: ${f} loaded!`);
        client.commands.set(props.help.name, props);
        props.help.aliases.forEach(alias => {
        client.aliases.set(alias, props.help.name);
        });
    });
});

client.load = (name) => {
    try {
        let props = require(`../commands/${name}`);
        if (props.init) props.init(client);
        client.commands.set(name, props);
        props.help.aliases.forEach(alias => {
            client.aliases.set(alias, props.help.name);
        });
        return false;
    } catch (err) {
        return [console.error(err)];
    }
};

client.unload = async (name) => {
    try {
        if (!name) return `Cannot find \`${name}\` command`;
        if (name.shutdown) await name.shutdown(client);
        delete require.cache[require.resolve(`../commands/${name}.js`)];
        return false;
    } catch (err) {
        return [console.error(err)];
    }
};

process.on('SIGINT', async function() {
    console.log("Shutting down...");
    await client.destroy();
    process.exit(1);
});